/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "fxCommonIncludes.fx"

// shader regs
float4 PolyBlend : register(PSC_POLYBLEND);
float4 shadelight : register(PSC_SHADELIGHT);
float3 shadevector : register(PSC_SHADEVECTOR);
float3 LightColor : register(PSC_LIGHTCOLOR);
float LightRadius : register(PSC_LIGHTRADIUS);
float4 Alpha : register(PSC_ALPHAVAL);
float2 WarpTime : register(PSC_WARPTIME);
float Gamma : register(PSC_GAMMA);
float Intensity : register(PSC_INTENSITY);
float Brightness : register(PSC_BRIGHTNESS);

// declare once and forget!!!
sampler tmu0Sampler : register(s0);
sampler tmu1Sampler : register(s1);
sampler tmu2Sampler : register(s2);
sampler tmu3Sampler : register(s3);
sampler tmu4Sampler : register(s4);
sampler tmu5Sampler : register(s5);
sampler tmu6Sampler : register(s6);
sampler tmu7Sampler : register(s7);


float4 ApplyGammaPS (float4 colorin, float texturescale)
{
	// it's faster to just do the calculation even if the values are 1
	return float4 (pow (abs (colorin.rgb * texturescale), Gamma) * Brightness, colorin.a);
}


// this ensures that we use the same dynamic calcs on surfs and meshs
float4 GenericDynamicPS (PS_GENERICDYNAMIC ps_in) : COLOR0
{
	// this clip is sufficient to exclude unlit portions; Add below may still bring it to 0
	// but in practice it's rare and it runs faster without a second clip
	clip ((LightRadius * LightRadius) - dot (ps_in.LightVector, ps_in.LightVector));

	// reading the diffuse texture early so that it should interleave with some ALU ops
	float4 diff = ApplyGammaPS (tex2D (tmu0Sampler, ps_in.TexCoord), Intensity);

		// this calc isn't correct per-theory but it matches with the calc used by light.exe and qrad.exe
		float Angle = ((dot (normalize (ps_in.Normal), normalize (ps_in.LightVector)) * 0.5f) + 0.5f) / 256.0f;

	// using our own custom attenuation, again it's not correct per-theory but matches the Quake tools
	float Add = max ((LightRadius - length (ps_in.LightVector)) * Angle, 0.0f);

	return float4 (diff.rgb * LightColor * Add, 0.0f);
}

