/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

struct VS_SURFCOMMON
{
	float4 Position: POSITION0;
	float2 TexCoord0: TEXCOORD0;
	float2 TexCoord1: TEXCOORD1;
	float Scroll : TEXCOORD2;
	int4 Styles: TEXCOORD3;
	float3 Normal : NORMAL0;
};

struct PS_SURFLIGHTMAPPED
{
	float4 Position: POSITION0;
	float2 TexCoord0: TEXCOORD0;
	float2 TexCoord1: TEXCOORD1;
	float Style: TEXCOORD2;
};

struct PS_SURFCOMMON
{
	float4 Position: POSITION0;
	float2 TexCoord0: TEXCOORD0;
	float2 TexCoord1: TEXCOORD1;
};

struct SKYVERTEX
{
	float4 Position: POSITION0;
	float3 TexCoord: TEXCOORD0;
};

#ifdef VERTEXSHADER
#include "fxVSIncludes.fx"

PS_SURFLIGHTMAPPED SurfLightmappedVS (VS_SURFCOMMON vs_in)
{
	PS_SURFLIGHTMAPPED vs_out;

	vs_out.Position = GetPosition (EntityMatrix, vs_in.Position);
	vs_out.TexCoord0 = vs_in.TexCoord0 + Scroll * vs_in.Scroll;
	vs_out.TexCoord1 = vs_in.TexCoord1;
	vs_out.Style = LightStyleValues[vs_in.Styles[SurfacePass]];

	return vs_out;
}

PS_GENERICDYNAMIC SurfDynamicVS (VS_SURFCOMMON vs_in)
{
	return GenericDynamicVS (
		vs_in.Position,
		vs_in.Normal,
		vs_in.TexCoord0 + Scroll * vs_in.Scroll
	);
}

PS_SURFCOMMON SurfTurbVS (VS_SURFCOMMON vs_in)
{
	PS_SURFCOMMON vs_out;

	vs_out.Position = GetPosition (EntityMatrix, vs_in.Position);
	vs_out.TexCoord0 = vs_in.TexCoord0 + Scroll * vs_in.Scroll;
	vs_out.TexCoord1 = vs_in.TexCoord1 + WarpTime;

	return vs_out;
}

PS_SURFCOMMON SurfAlphaVS (VS_SURFCOMMON vs_in)
{
	PS_SURFCOMMON vs_out;

	vs_out.Position = GetPosition (EntityMatrix, vs_in.Position);
	vs_out.TexCoord0 = vs_in.TexCoord0 + Scroll * vs_in.Scroll;
	vs_out.TexCoord1 = vs_in.TexCoord1;

	return vs_out;
}

SKYVERTEX SurfSkyVS (VS_SURFCOMMON vs_in)
{
	SKYVERTEX vs_out;

	vs_out.Position = GetPosition (EntityMatrix, vs_in.Position);
	vs_out.TexCoord = mul (SkyMatrix, mul (LocalMatrix, vs_in.Position)).xyz;

	return vs_out;
}
#endif


#ifdef PIXELSHADER
#include "fxPSIncludes.fx"

#define surfSampler tmu0Sampler
#define lmapSampler tmu1Sampler
#define skySampler tmu2Sampler


float4 SurfLightmappedPS (PS_SURFLIGHTMAPPED ps_in) : COLOR0
{
	float4 diff = tex2D (surfSampler, ps_in.TexCoord0);
	float4 lmap = tex2D (lmapSampler, ps_in.TexCoord1);

	return ApplyGammaPS (diff, Intensity) * lmap * ps_in.Style;
}

float4 SurfAlphaPS (PS_SURFCOMMON ps_in) : COLOR0
{
	return ApplyGammaPS (float4 (tex2D (surfSampler, ps_in.TexCoord0).rgb, Alpha.a), 1.0f);
}

float4 SurfDynamicPS (PS_GENERICDYNAMIC ps_in) : COLOR0
{
	return GenericDynamicPS (ps_in);
}

float4 SurfTurbPS (PS_SURFCOMMON ps_in) : COLOR0
{
	return ApplyGammaPS (float4 (
		tex2Dgrad (
			surfSampler,
			ps_in.TexCoord0 + sin (ps_in.TexCoord1) * 0.125f,
			ddx (ps_in.TexCoord0),
			ddy (ps_in.TexCoord0)
		).rgb, Alpha.a
	), 1.0f);
}

struct SKYFRAGMENT
{
	float4 Color : COLOR0;
	float Depth : DEPTH0;
};

SKYFRAGMENT SurfSkyPS (SKYVERTEX ps_in)
{
	SKYFRAGMENT ps_out;

	ps_out.Color = ApplyGammaPS (texCUBE (skySampler, ps_in.TexCoord), 1.0f);
	ps_out.Depth = 1.0f;

	return ps_out;
}
#endif

