/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "fxCommonIncludes.fx"

// entity transform in world and local space
float4x4 WorldMatrix : register(VSC_WORLDMATRIX);
float4x4 EntityMatrix : register(VSC_ENTITYMATRIX);
float4x4 LocalMatrix : register(VSC_LOCALMATRIX);
float WarpTime : register(VSC_WARPTIME);
float3 vup : register(VSC_VUP);
float3 vpn : register(VSC_VPN);
float3 vright : register(VSC_VRIGHT);
float3 r_origin : register(VSC_RORIGIN);
float3 entorigin : register(VSC_SPRITEORIGIN);
float2 Scroll : register(VSC_TEXSCROLL);
float4x4 SkyMatrix : register(VSC_SKYMATRIX);
float3 TransformedLight : register(VSC_TRANSFORMEDDL);
float SurfacePass : register(VSC_SURFACEPASS);
float4x4 OrthoMatrix : register(VSC_ORTHOMATRIX);
float2 RTInverseSize : register(VSC_RTINVERSE);

// mesh
float3 MeshMove : register(VSC_MESHMOVE);
float3 MeshFrontV : register(VSC_MESHFRONTV);
float3 MeshBackV : register(VSC_MESHBACKV);
float MeshSuitScale : register(VSC_MESHSUITSCALE);
float MeshBlend : register(VSC_MESHBLEND);

// this reg is kept high to keep the lower ones free for the dynamic per-obj stuff
float4 LightStyles[MAX_PACKED_LIGHTSTYLES] : register(VSC_LIGHTSTYLES);

// this is a packed version of the lightstyles array for direct reading
static float LightStyleValues[MAX_LIGHTSTYLES] = (float[MAX_LIGHTSTYLES]) LightStyles;

// used so we can call GetPosition on already-transformed input
static const float4x4 identityMatrix = {1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1};

// http://aras-p.info/blog/2016/04/08/solving-dx9-half-pixel-offset/
float4 GetPosition (float4x4 inMatrix, float4 inPosition)
{
	float4 outPosition = mul (inMatrix, inPosition);
	outPosition.xy += RTInverseSize.xy * outPosition.w;
	return outPosition;
}


// this ensures that we use the same dynamic calcs on surfs and meshs
PS_GENERICDYNAMIC GenericDynamicVS (float4 Position, float3 Normal, float2 TexCoord)
{
	PS_GENERICDYNAMIC vs_out;

	vs_out.Position = GetPosition (EntityMatrix, Position);
	vs_out.TexCoord = TexCoord;
	vs_out.Normal = Normal;
	vs_out.LightVector = TransformedLight - Position.xyz;

	return vs_out;
}

