/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// models.c -- model loading and caching

#include "gl_local.h"

extern model_t	*loadmodel;
extern int		modfilelen;

/*
==============================================================================

ALIAS MODELS

==============================================================================
*/

/*
=================
Mod_LoadAliasModel
=================
*/
void Mod_LoadAliasModel (model_t *mod, void *buffer)
{
	dmdl_t *pinmodel = (dmdl_t *) buffer;
	int version = LittleLong (pinmodel->version);

	if (version != ALIAS_VERSION)
		ri->Sys_Error (ERR_DROP, "%s has wrong version number (%i should be %i)", mod->name, version, ALIAS_VERSION);

	if (pinmodel->skinheight > MAX_LBM_HEIGHT) ri->Sys_Error (ERR_DROP, "model %s has a skin taller than %d", mod->name, MAX_LBM_HEIGHT);
	if (pinmodel->num_xyz <= 0) ri->Sys_Error (ERR_DROP, "model %s has no vertices", mod->name);
	if (pinmodel->num_xyz > MAX_VERTS) ri->Sys_Error (ERR_DROP, "model %s has too many vertices", mod->name);
	if (pinmodel->num_st <= 0) ri->Sys_Error (ERR_DROP, "model %s has no st vertices", mod->name);
	if (pinmodel->num_tris <= 0) ri->Sys_Error (ERR_DROP, "model %s has no triangles", mod->name);
	if (pinmodel->num_frames <= 0) ri->Sys_Error (ERR_DROP, "model %s has no frames", mod->name);

	// cut right down
	aliashdr_t *pheader = ri->Hunk_Alloc (sizeof (aliashdr_t));

	// load the frames (these are all that are kept in memory and just for the scale and translate values;
	// we could even get rid of that and roll them into the vertex buffer but we also need them for culling)
	pheader->frames = (maliasframe_t *) ri->Hunk_Alloc (pinmodel->num_frames * sizeof (maliasframe_t));
	pheader->numframes = LittleLong (pinmodel->num_frames);

	for (int i = 0; i < pinmodel->num_frames; i++)
	{
		daliasframe_t *pinframe = (daliasframe_t *) ((byte *) pinmodel + pinmodel->ofs_frames + i * pinmodel->framesize);
		maliasframe_t *poutframe = &pheader->frames[i];

		for (int j = 0; j < 3; j++)
		{
			poutframe->scale[j] = LittleFloat (pinframe->scale[j]);
			poutframe->translate[j] = LittleFloat (pinframe->translate[j]);
		}
	}

	// load the skins (these just need their names kept in memory for the registration)
	pheader->numskins = pinmodel->num_skins;

	for (int i = 0; i < pinmodel->num_skins; i++)
	{
		char *skin = (char *) pinmodel + pinmodel->ofs_skins + i * MAX_SKINNAME;

		pheader->skins[i] = (char *) ri->Hunk_Alloc (strlen (skin) + 1);
		mod->skins[i] = R_FindImage (strcpy (pheader->skins[i], skin), it_skin);
	}

	mod->type = mod_alias;

	Vector3Set (mod->mins, -32, -32, -32);
	Vector3Set (mod->maxs, 32, 32, 32);

	// now use the in-memory version of the model to create the mesh buffers so that we don't need to keep
	// the mesh buffer data in-memory beyond load time
	R_CreateMeshBuffers (mod, pinmodel);
}

