/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "gl_local.h"
#include "resource.h"

IDirect3DVertexDeclaration9 *d3d_BBoxDecl = NULL;
IDirect3DVertexShader9 *d3d_BBoxVS = NULL;
IDirect3DPixelShader9 *d3d_BBoxPS = NULL;

void BBox_OnCreate (void)
{
	D3DVERTEXELEMENT9 layout[] = {
		VDECL (0, 0, D3DDECLTYPE_FLOAT3, D3DDECLUSAGE_POSITION, 0),
		D3DDECL_END ()
	};

	d3d_BBoxDecl = R_CreateVertexDeclaration (layout);
	R_CreateShaders (BBoxVS, BBoxPS, IDR_BBOXSHADER);
}


void BBox_OnReset (void) {}
void BBox_OnLost (void) {}
void BBox_OnDestroy (void) {}

void BBox_Register (void)
{
	static qboolean registered = false;

	if (!registered)
	{
		R_RegisterHandler (BBox_OnCreate, BBox_OnLost, BBox_OnReset, BBox_OnDestroy);
		registered = true;
	}
}


void R_DrawBBoxBox (vec3_t *boxpoints)
{
	R_SetVertexDeclaration (d3d_BBoxDecl);
	R_SetVertexShader (d3d_BBoxVS);
	R_SetPixelShader (d3d_BBoxPS);
	R_SetDPUPMode ();

	R_SetRenderStates (0);// RS_DEPTHTEST | RS_WIREFRAME);

	unsigned short boxindices[36] = {
		0, 1, 2, 0, 2, 3,
		0, 3, 7, 0, 7, 4,
		0, 4, 5, 0, 5, 1,
		6, 2, 1, 6, 1, 5,
		6, 5, 4, 6, 4, 7,
		6, 7, 3, 6, 3, 2
	};

	d3d_Device->lpVtbl->DrawIndexedPrimitiveUP (
		d3d_Device,
		D3DPT_TRIANGLELIST,
		0,
		8,
		12,
		boxindices,
		D3DFMT_INDEX16,
		boxpoints,
		sizeof (vec3_t));

	c_draw_call++;
}


void R_DrawBBoxMinMaxs (float *mins, float *maxs)
{
	float bbexpand = 2;

	// compute a full bounding box
	vec3_t bbox[8] = {
		{maxs[0] + bbexpand, maxs[1] + bbexpand, maxs[2] + bbexpand},
		{maxs[0] + bbexpand, maxs[1] + bbexpand, mins[2] - bbexpand},
		{maxs[0] + bbexpand, mins[1] - bbexpand, mins[2] - bbexpand},
		{maxs[0] + bbexpand, mins[1] - bbexpand, maxs[2] + bbexpand},
		{mins[0] - bbexpand, maxs[1] + bbexpand, maxs[2] + bbexpand},
		{mins[0] - bbexpand, maxs[1] + bbexpand, mins[2] - bbexpand},
		{mins[0] - bbexpand, mins[1] - bbexpand, mins[2] - bbexpand},
		{mins[0] - bbexpand, mins[1] - bbexpand, maxs[2] + bbexpand}
	};

	R_DrawBBoxBox (bbox);
}

