/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "gl_local.h"
#include "glw_win.h"

// engine specific stuff (this will all move to gl_rinit.c)
#define	WINDOW_CLASS_NAME	"DirectQIIRedux"

glwstate_t glw_state;


void Misc_OnCreate (void) {}
void Misc_OnReset (void) {}
void Misc_OnLost (void) {}
void Misc_OnDestroy (void) {}

void Misc_Register (void)
{
	static qboolean registered = false;

	if (!registered)
	{
		R_RegisterHandler (Misc_OnCreate, Misc_OnLost, Misc_OnReset, Misc_OnDestroy);
		registered = true;
	}
}


/*
===================
GL_Strings_f
===================
*/
void GL_Strings_f (void)
{
	D3DADAPTER_IDENTIFIER9 Identifier;

	d3d_Object->lpVtbl->GetAdapterIdentifier (d3d_Object, 0, 0, &Identifier);

	ri->Con_Printf (PRINT_ALL, "\n");
	ri->Con_Printf (PRINT_ALL, "Driver: %s\n", Identifier.Driver);
	ri->Con_Printf (PRINT_ALL, "Description: %s\n", Identifier.Description);
	ri->Con_Printf (PRINT_ALL, "Device: %s\n", Identifier.DeviceName);
}


/*
==================
R_InitTextures
==================
*/
void R_InitTextures (void)
{
	// data that will be used for textures
	unsigned notexture[] = {0xffffffff, 0xff000000, 0xff000000, 0xffffffff};
	unsigned blacktexture[] = {0xff000000, 0xff000000, 0xff000000, 0xff000000};
	unsigned greytexture[] = {0xff808080, 0xff808080, 0xff808080, 0xff808080};
	unsigned whitetexture[] = {0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff};

	// and load them
	r_notexture = R_LoadPic ("***r_notexture***", (byte *) notexture, 2, 2, it_wall, NULL);
	r_blacktexture = R_LoadPic ("***r_blacktexture***", (byte *) blacktexture, 2, 2, it_wall, NULL);
	r_greytexture = R_LoadPic ("***r_greytexture***", (byte *) greytexture, 2, 2, it_wall, NULL);
	r_whitetexture = R_LoadPic ("***r_whitetexture***", (byte *) whitetexture, 2, 2, it_wall, NULL);
}


/*
===================
R_SetMode
===================
*/
rserr_t R_SetMode (int *pwidth, int *pheight, int mode, qboolean fullscreen)
{
	int width, height;
	const char *win_fs[] = {"W", "FS"};

	ri->Con_Printf (PRINT_ALL, "Initializing Direct3D display\n");
	ri->Con_Printf (PRINT_ALL, "...setting mode %d:", mode);

	if (!R_GetVideoModeInfo (&width, &height, mode))
	{
		ri->Con_Printf (PRINT_ALL, " invalid mode\n");
		return rserr_invalid_mode;
	}

	ri->Con_Printf (PRINT_ALL, " %d %d %s\n", width, height, win_fs[fullscreen]);

	// the window is always created as a standard "windowed mode" window and d3d device creation will switch it to fullscreen if needed
	int exstyle = 0;
	int stylebits = WINDOW_STYLE;
	RECT r;

	SetRect (&r, 0, 0, width, height);
	AdjustWindowRect (&r, stylebits, FALSE);

	glw_state.hWnd = CreateWindowEx (
		exstyle,
		WINDOW_CLASS_NAME,
		WINDOW_CLASS_NAME,
		stylebits,
		ri->vars->xpos->value,
		ri->vars->ypos->value,
		r.right - r.left,
		r.bottom - r.top,
		NULL,
		NULL,
		GetModuleHandle (NULL),
		NULL);

	if (!glw_state.hWnd)
		ri->Sys_Error (ERR_FATAL, "Couldn't create window");

	ShowWindow (glw_state.hWnd, SW_SHOW);
	UpdateWindow (glw_state.hWnd);

	// let the sound and input subsystems know about the new window
	ri->Vid_NewWindow (width, height);

	// store out width and height
	// (fixme - this really should come from reading the backbuffer)
	*pwidth = width;
	*pheight = height;

	return rserr_ok;
}


void R_LoadGfx (void)
{
	// re-init all gfx objects when going to a new game
	// set default render states
	R_SetDefaultState ();

	// init anything else that needs to be inited after the renderer is up
	R_InitImages ();
	Mod_Init ();
	R_InitTextures ();
	Draw_InitLocal ();
}


void R_UnloadGfx (void)
{
	// unload current gfx objects when going to a new game
	R_ShutdownImages ();
	Mod_FreeAll ();

	// force these to free with a known-bad registration sequence
	R_FreeUnusedMeshBuffers (-1);
	R_FreeUnusedSpriteBuffers (-1);
	R_ReleaseLightmaps (-1);
}


/*
===============
R_Init
===============
*/
int R_Init (void)
{
	// create the window and set up the context
	// this needs to be called *after* we create d3d and get the modes so that we'll have the mode info for setting up our window
	if (R_SetMode (&vid.width, &vid.height, ri->vars->mode->value, false) != rserr_ok)
	{
		R_ShutdownVideo ();
		ri->Con_Printf (PRINT_ALL, "ref_gl::R_Init() - could not R_SetMode()\n");
		return -1;
	}

	// set up the palette
	Draw_GetPalette ();

	// get our cvars and commands
	r_lefthand = ri->Cvar_Get ("hand", "0", CVAR_USERINFO | CVAR_ARCHIVE);
	r_norefresh = ri->Cvar_Get ("r_norefresh", "0", 0);
	r_drawentities = ri->Cvar_Get ("r_drawentities", "1", 0);
	r_drawworld = ri->Cvar_Get ("r_drawworld", "1", 0);
	r_novis = ri->Cvar_Get ("r_novis", "0", 0);
	r_nocull = ri->Cvar_Get ("r_nocull", "0", 0);
	r_lerpmodels = ri->Cvar_Get ("r_lerpmodels", "1", 0);
	r_speeds = ri->Cvar_Get ("r_speeds", "0", 0);
	r_lightlevel = ri->Cvar_Get ("r_lightlevel", "0", 0);
	r_showbboxes = ri->Cvar_Get ("r_showbboxes", "0", 0);
	r_shownullmodels = ri->Cvar_Get ("r_shownullmodels", "0", 0);
	r_fullbright = ri->Cvar_Get ("r_fullbright", "0", 0);

	vid_gamma = ri->Cvar_Get ("vid_gamma", "1.0", CVAR_ARCHIVE);
	vid_brightness = ri->Cvar_Get ("brightness", "1.0", CVAR_ARCHIVE);

	// these 2 vars originally behaved differently; gl_modulate was a multiplier for lightstyles, whereas intensity was a multiplier for
	// textures and which was clamped at 255.  in this engine with no more clamping they both do the same thing.  they're retained as 
	// separate cvars only in case a player already has values for them set.
	gl_modulate = ri->Cvar_Get ("gl_modulate", "1", CVAR_ARCHIVE);
	intensity = ri->Cvar_Get ("intensity", "2.0", CVAR_ARCHIVE);

	gl_dynamic = ri->Cvar_Get ("gl_dynamic", "1", 0);
	gl_clear = ri->Cvar_Get ("gl_clear", "0", 0);
	gl_cull = ri->Cvar_Get ("gl_cull", "1", 0);
	gl_polyblend = ri->Cvar_Get ("gl_polyblend", "1", 0);
	gl_texturemode = ri->Cvar_Get ("gl_texturemode", "GL_LINEAR_MIPMAP_LINEAR", CVAR_ARCHIVE);
	gl_textureanisotropy = ri->Cvar_Get ("gl_textureanisotropy", "1", CVAR_ARCHIVE);
	gl_lockpvs = ri->Cvar_Get ("gl_lockpvs", "0", 0);
	gl_lockfrustum = ri->Cvar_Get ("gl_lockfrustum", "0", 0);

	ri->Cmd_AddCommand ("imagelist", R_ImageList_f);
	ri->Cmd_AddCommand ("screenshot", R_ScreenShot_f);
	ri->Cmd_AddCommand ("modellist", Mod_Modellist_f);
	ri->Cmd_AddCommand ("gl_strings", GL_Strings_f);

	// and the device can't be created until *after* we've created the window
	if (!R_InitVideo (glw_state.hWnd))
	{
		R_ShutdownVideo ();
		ri->Con_Printf (PRINT_ALL, "ref_gl::R_Init() - could not load driver\n");
		return -1;
	}

	// load Gfx objects
	R_LoadGfx ();

	return 1;
}

/*
===============
R_Shutdown
===============
*/
void R_Shutdown (void)
{
	ri->Cmd_RemoveCommand ("modellist");
	ri->Cmd_RemoveCommand ("screenshot");
	ri->Cmd_RemoveCommand ("imagelist");
	ri->Cmd_RemoveCommand ("gl_strings");

	// unload Gfx objects
	R_UnloadGfx ();

	// release shaders
	R_ReleaseShaders ();

	// shutdown our QGL subsystem
	R_ShutdownVideo ();

	// shut down OS specific OpenGL stuff like contexts, etc.
	// to do - ensure that we've run through d3d device shutdown first...
	if (glw_state.hWnd)
	{
		DestroyWindow (glw_state.hWnd);
		glw_state.hWnd = NULL;
	}

	UnregisterClass (WINDOW_CLASS_NAME, GetModuleHandle (NULL));
}


/*
===================
R_AppActivate
===================
*/
void R_AppActivate (qboolean active)
{
	if (active)
	{
		ShowWindow (glw_state.hWnd, SW_RESTORE);
	}
	else
	{
		if (ri->vars->fullscreen->value)
			ShowWindow (glw_state.hWnd, SW_MINIMIZE);
	}
}


// imports for the ref API
void R_BeginRegistration (char *map);
struct model_s *R_RegisterModel (char *name);
struct image_s *R_RegisterSkin (char *name);
void R_SetSky (char *name, float rotate, vec3_t axis);
void R_EndRegistration (void);
void R_RenderFrame (refdef_t *rd);
struct image_s *Draw_FindPic (char *name);
void Draw_Pic (int x, int y, char *name);
void Draw_Char (int x, int y, int c);
void Draw_String (int x, int y, const char *str, int len, int xor);
void Draw_Field (int x, int y, int color, int width, int value);
void Draw_Fill (int x, int y, int w, int h, int c);
void Draw_FadeScreen (void);
void Draw_Clear (unsigned colour);
void R_HandleAltEnter (void);
char *R_NameForImage (struct image_s *image);


/*
===================
GetRefAPI

===================
*/
refexport_t *GetRefAPI (refimport_t *rimp)
{
	static refexport_t rexp;

	ri = rimp;

	rexp.api_version = API_VERSION;
	rexp.NameForImage = R_NameForImage;

	rexp.palette = d_8to24table;

	rexp.BeginRegistration = R_BeginRegistration;
	rexp.RegisterModel = R_RegisterModel;
	rexp.RegisterSkin = R_RegisterSkin;
	rexp.RegisterPic = Draw_FindPic;
	rexp.SetSky = R_SetSky;
	rexp.EndRegistration = R_EndRegistration;

	rexp.RenderFrame = R_RenderFrame;

	rexp.DrawGetPicSize = Draw_GetPicSize;
	rexp.DrawPic = Draw_Pic;
	rexp.DrawStretchPic = Draw_StretchPic;
	rexp.DrawConback = Draw_ConsoleBackground;
	rexp.DrawChar = Draw_Char;
	rexp.DrawString = Draw_String;
	rexp.DrawField = Draw_Field;
	rexp.DrawFill = Draw_Fill;
	rexp.DrawFadeScreen = Draw_FadeScreen;
	rexp.Clear = Draw_Clear;
	rexp.LoadPCX = R_LoadPCX;

	rexp.DrawStretchRaw = Draw_StretchRaw;

	rexp.Init = R_Init;
	rexp.Shutdown = R_Shutdown;

	rexp.SetCinematicPalette = R_SetCinematicPalette;
	rexp.BeginFrame = R_BeginFrame;
	rexp.Set2DMode = R_SetGL2D;
	rexp.EndFrame = R_EndFrame;

	rexp.LoadGfx = R_LoadGfx;
	rexp.UnloadGfx = R_UnloadGfx;

	rexp.AppActivate = R_AppActivate;
	rexp.HandleAltEnter = R_HandleAltEnter;
	rexp.ForceVidRestart = R_ModifyVidCvars;

	Swap_Init ();

	// create d3d and get video modes (if this fails we'll detect it at device creation time)
	if (!R_CreateDirect3D ())
	{
		R_ShutdownVideo ();
		ri->Sys_Error (ERR_FATAL, "Failed to initialize D3D");
	}

	return &rexp;
}


